@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2023, University of Amsterdam
+    Copyright (c)  1985-2025, University of Amsterdam
                               VU University Amsterdam
                               CWI, Amsterdam
                               SWI-Prolog Solutions b.v.
@@ -39,7 +39,7 @@
           [ '$find_library'/5,
             '$in_library'/3,
             '$define_predicate'/1,
-            '$update_library_index'/0,
+            '$update_library_index'/1,          % +Options
             '$autoload'/1,
 
             make_library_index/1,
@@ -71,6 +71,8 @@
 user:file_search_path(autoload, swi(library)).
 user:file_search_path(autoload, pce(prolog/lib)).
 user:file_search_path(autoload, app_config(lib)).
+user:file_search_path(autoload, Dir) :-
+    '$ext_library_directory'(Dir).
 
 :- create_prolog_flag(warn_autoload, false, []).
 
@@ -134,15 +136,20 @@ user:file_search_path(autoload, app_config(lib)).
 :- thread_local
     silent/0.
 
-%!  '$update_library_index'
+%!  '$update_library_index'(+Options)
 %
 %   Called from make/0 to update the index   of the library for each
 %   library directory that has a writable   index.  Note that in the
 %   Windows  version  access_file/2  is  mostly   bogus.  We  assert
-%   silent/0 to suppress error messages.
+%   silent/0 to suppress error messages.  Options:
+%
+%       - system(+Boolean)
+%         Do (not) include system libraries.   Default `false`.
+%       - user(+Boolean)
+%         Do (not) include user libraries.  Default `true`.
 
-'$update_library_index' :-
-    setof(Dir, writable_indexed_directory(Dir), Dirs),
+'$update_library_index'(Options) :-
+    setof(Dir, writable_indexed_directory(Dir, Options), Dirs),
     !,
     setup_call_cleanup(
         asserta(silent, Ref),
@@ -152,7 +159,7 @@ user:file_search_path(autoload, app_config(lib)).
     ->  reload_library_index
     ;   true
     ).
-'$update_library_index'.
+'$update_library_index'(_).
 
 guarded_make_library_index([]).
 guarded_make_library_index([Dir|Dirs]) :-
@@ -163,11 +170,19 @@ guarded_make_library_index([Dir|Dirs]) :-
     ),
     guarded_make_library_index(Dirs).
 
-%!  writable_indexed_directory(-Dir) is nondet.
+%!  writable_indexed_directory(-Dir, +Options) is nondet.
 %
 %   True when Dir is an indexed   library  directory with a writable
 %   index, i.e., an index that can be updated.
 
+writable_indexed_directory(Dir, Options) :-
+    current_prolog_flag(home, Home),
+    writable_indexed_directory(Dir),
+    (   sub_atom(Dir, 0, _, _, Home)
+    ->  '$option'(system(true), Options, false)
+    ;   '$option'(user(true), Options, true)
+    ).
+
 writable_indexed_directory(Dir) :-
     index_file_name(IndexFile, autoload('INDEX'), [access([read,write])]),
     file_directory_name(IndexFile, Dir).
@@ -356,7 +371,7 @@ make_library_index2(Dir, Patterns) :-
     pattern_files(Patterns, DirS, Files),
     (   library_index_out_of_date(Dir, AbsIndex, Files)
     ->  do_make_library_index(AbsIndex, DirS, Files),
-        flag('$modified_index', _, true)
+        set_flag('$modified_index', true)
     ;   true
     ).
 
@@ -388,10 +403,10 @@ library_index_out_of_date(_Dir, Index, _Files) :-
 library_index_out_of_date(Dir, Index, Files) :-
     time_file(Index, IndexTime),
     (   time_file(Dir, DotTime),
-        DotTime > IndexTime
-    ;   '$member'(File, Files),
+        DotTime - IndexTime > 0.001             % compensate for jitter
+    ;   '$member'(File, Files),                 % and rounding
         time_file(File, FileTime),
-        FileTime > IndexTime
+        FileTime - IndexTime > 0.001
     ),
     !.
 
@@ -454,6 +469,11 @@ index_header(Fd):-
     format(Fd, '    Purpose: Provide index for autoload~n', []),
     format(Fd, '*/~n~n', []).
 
+%!  exports(+File, -Module, -Exports) is det.
+%
+%   Get the exports from a library as a list of PIs.
+
+:- public exports/3.                            % using by library(prolog_deps).
 exports(File, Module, Exports) :-
     (   current_prolog_flag(xref, Old)
     ->  true
@@ -461,16 +481,17 @@ exports(File, Module, Exports) :-
     ),
     setup_call_cleanup(
         set_prolog_flag(xref, true),
-        exports_(File, Module, Exports),
+        snapshot(exports_(File, Module, Exports)),
         set_prolog_flag(xref, Old)).
 
 exports_(File, Module, Exports) :-
     State = state(true, _, []),
-    (   '$source_term'(File, _,_,Term0,_,_,[syntax_errors(quiet)]),
-        (   is_list(Term0)
-        ->  '$member'(Term, Term0)
-        ;   Term = Term0
-        ),
+    (   '$source_term'(File,
+                       _Read,_RLayout,
+                       Term,_TermLayout,
+                       _Stream,
+                       [ syntax_errors(quiet)
+                       ]),
         (   Term = (:- module(M,Public)),
             is_list(Public),
             arg(1, State, true)
@@ -480,10 +501,10 @@ exports_(File, Module, Exports) :-
             fail
         ;   nb_setarg(1, State, false),
             fail
-        ;   Term = (:- export(PI)),
-            ground(PI)
-        ->  arg(3, State, E0),
-            '$append'(E0, [PI], E1),
+        ;   Term = (:- export(Export))
+        ->  phrase(export_pi(Export), PIs),
+            arg(3, State, E0),
+            '$append'(E0, PIs, E1),
             nb_setarg(3, State, E1),
             fail
         ;   Term = (:- use_foreign_library(Lib)),
@@ -495,6 +516,8 @@ exports_(File, Module, Exports) :-
         ;   Term = (:- Directive),
             nonvar(Directive)
         ->  fail
+        ;   Term == []                          % Expansion for conditionals
+        ->  fail
         ;   !
         )
     ;   true
@@ -502,6 +525,17 @@ exports_(File, Module, Exports) :-
     arg(2, State, Module),
     arg(3, State, Exports).
 
+export_pi(Var) -->
+    { var(Var) },
+    !.
+export_pi((A,B)) -->
+    !,
+    export_pi(A),
+    export_pi(B).
+export_pi(PI) -->
+    { ground(PI) },
+    [PI].
+
 
                  /*******************************
                  *            EXTENDING         *
@@ -573,7 +607,7 @@ restore_sandbox(Sandboxed) :-
     autoload_from(PI, LoadModule, FullFile),
     do_autoload(FullFile, PI, LoadModule).
 
-%!  autoload_from(+PI, -LoadModule, -File) is semidet.
+%!  autoload_from(+PI, -LoadModule, -PlFile) is semidet.
 %
 %   True when PI can be defined  by   loading  File which is defined the
 %   module LoadModule.
@@ -614,7 +648,7 @@ autoload_in(user_or_explicit, explicit, _).
 autoload_in(user_or_explicit, _,        user).
 
 
-%!  do_autoload(+File, :PI, +LoadModule) is det.
+%!  do_autoload(Library, :PI, +LoadModule) is det.
 %
 %   Load File, importing PI into the qualified  module. File is known to
 %   define LoadModule. There are three cases:
@@ -626,19 +660,23 @@ autoload_in(user_or_explicit, _,        user).
 %       verifies the predicate really exists, but doesn't validate
 %       that it is defined.
 %     - We must load the module and import the target predicate.
+%
+%   @arg Library is an absolute file   name, either without extension or
+%   with the source (.pl) extension.
 
 do_autoload(Library, Module:Name/Arity, LoadModule) :-
     functor(Head, Name, Arity),
     '$update_autoload_level'([autoload(true)], Old),
     verbose_autoload(Module:Name/Arity, Library),
+    loadable_file(Library, File),
     '$compilation_mode'(OldComp, database),
     (   Module == LoadModule
-    ->  ensure_loaded(Module:Library)
+    ->  ensure_loaded(Module:File)
     ;   (   '$c_current_predicate'(_, LoadModule:Head),
             '$get_predicate_attribute'(LoadModule:Head, defined, 1),
             \+ '$loading'(Library)
         ->  Module:import(LoadModule:Name/Arity)
-        ;   use_module(Module:Library, [Name/Arity])
+        ;   use_module(Module:File, [Name/Arity])
         ),
         warn_autoload(Module, LoadModule:Name/Arity)
     ),
@@ -646,6 +684,14 @@ do_autoload(Library, Module:Name/Arity, LoadModule) :-
     '$set_autoload_level'(Old),
     '$c_current_predicate'(_, Module:Head).
 
+loadable_file(PlFile, File) :-
+    exists_file(PlFile), !,
+    File = PlFile.
+loadable_file(PlFile, Base) :-
+    file_name_extension(Base, pl, PlFile),
+    !.
+loadable_file(File, File).
+
 verbose_autoload(PI, Library) :-
     current_prolog_flag(verbose_autoload, true),
     !,
@@ -710,7 +756,14 @@ goal_name_arity(Head, Head, 0).
 
 %!  library_info(+Spec, +AutoloadContext, -FullFile, -Module, -Exports)
 %
-%   Find information about a library.
+%   Find information about a library. Spec  is the file specification as
+%   it appears in the  autoload/1,2  call.   AutoloadContext  is  a term
+%   File:Line, providing the location of the directive.
+%
+%   @arg FullFile is the source (.pl) file in canonical (absolute)
+%   notation.
+%   @arg Module is the module defined in FullFile
+%   @arg Exports is a list of predicate indicators.
 
 library_info(Spec, _, FullFile, Module, Exports) :-
     '$resolved_source_path'(Spec, FullFile, []),
@@ -718,32 +771,46 @@ library_info(Spec, _, FullFile, Module, Exports) :-
     (   \+ '$loading_file'(FullFile, _Queue, _LoadThread)
     ->  '$current_module'(Module, FullFile),
         '$module_property'(Module, exports(Exports))
-    ;   library_info_from_file(FullFile, Module, Exports)
+    ;   library_info_from_file(FullFile, _, Module, Exports)
     ).
 library_info(Spec, Context, FullFile, Module, Exports) :-
     (   Context = (Path:_Line)
     ->  Extra = [relative_to(Path)]
     ;   Extra = []
     ),
-    (   absolute_file_name(Spec, FullFile,
+    (   absolute_file_name(Spec, AbsFile,
                            [ file_type(prolog),
                              access(read),
                              file_errors(fail)
                            | Extra
                            ])
-    ->  '$register_resolved_source_path'(Spec, FullFile),
-        library_info_from_file(FullFile, Module, Exports)
+    ->  library_info_from_file(AbsFile, FullFile, Module, Exports),
+        '$register_resolved_source_path'(Spec, FullFile)
+    ;   absolute_file_name(Spec, FullFile,
+                           [ file_type(prolog),
+                             solutions(all),
+                             file_errors(fail)
+                           | Extra
+                           ]),
+        source_file(FullFile),
+        '$current_module'(Module, FullFile)
+    ->  '$module_property'(Module, exports(Exports))
     ;   autoload_error(Context, no_file(Spec)),
         fail
     ).
 
-library_info_from_file(FullFile, Module, Exports) :-
+library_info_from_file(QlfFile, PlFile, Module, Exports) :-
+    file_name_extension(_, qlf, QlfFile),
+    !,
+    '$qlf_module'(QlfFile, Info),
+    _{module:Module, exports:Exports, file:PlFile} :< Info.
+library_info_from_file(PlFile, PlFile, Module, Exports) :-
     setup_call_cleanup(
         '$set_source_module'(OldModule, system),
         setup_call_cleanup(
-            '$open_source'(FullFile, In, State, [], []),
+            '$open_source'(PlFile, In, State, [], []),
             '$term_in_file'(In, _Read, _RLayout, Term, _TLayout, _Stream,
-                            [FullFile], []),
+                            [PlFile], []),
             '$close_source'(State, true)),
         '$set_source_module'(OldModule)),
     (   Term = (:- module(Module, Exports))
@@ -813,16 +880,17 @@ materialize_autoload(Count) :-
 
 materialize_autoload(M, State) :-
     (   current_autoload(M:File, Context, Import),
-        library_info(File, Context, FullFile, _LoadModule, _Exports),
+        library_info(File, Context, PlFile, _LoadModule, _Exports),
         arg(1, State, N0),
         N is N0+1,
         nb_setarg(1, State, N),
+        loadable_file(PlFile, LoadFile),
         (   Import == all
-        ->  verbose_autoload(M:all, FullFile),
-            use_module(M:FullFile)
+        ->  verbose_autoload(M:all, PlFile),
+            use_module(M:LoadFile)
         ;   Import = import(Preds)
-        ->  verbose_autoload(M:Preds, FullFile),
-            use_module(M:FullFile, Preds)
+        ->  verbose_autoload(M:Preds, PlFile),
+            use_module(M:LoadFile, Preds)
         ),
         fail
     ;   true
@@ -887,6 +955,8 @@ valid_import_list([H0|T0], [H|T]) :-
 %
 %   Put an `autoload` flag on all   predicates declared using autoload/2
 %   to prevent duplicates or the user defining the same predicate.
+%
+%   @arg File is the first argument of autoload/1,2
 
 register_autoloads([], _, _, _).
 register_autoloads([PI|T], Module, File, Context) :-
@@ -932,10 +1002,15 @@ current_autoload(M:File, Context, Term) :-
 		 *            CHECK		*
 		 *******************************/
 
+%!  warn_autoload(+TargetModule, :PI) is det.
+%
+%   @arg PI is of the shape LoadModule:Name/Arity.
+
 warn_autoload(TargetModule, PI) :-
     current_prolog_flag(warn_autoload, true),
     \+ current_prolog_flag(xref, true),
     \+ nb_current('$autoload_warning', true),
+    \+ nowarn_autoload(TargetModule, PI),
     '$pi_head'(PI, Head),
     source_file(Head, File),
     expansion_hook(P),
@@ -953,6 +1028,25 @@ expansion_hook(user:goal_expansion(_,_,_,_)).
 expansion_hook(system:goal_expansion(_,_)).
 expansion_hook(system:goal_expansion(_,_,_,_)).
 
+%!  nowarn_autoload(+TargetModule, +LoadModulePI) is semidet.
+%
+%   True when LoadModule:'$nowarn_autoload'(PI,TargetModule)  is defined
+%   and true.
+%
+%   @tbd As is, these  facts  must  be   defined  by  the  library being
+%   autoloaded. Possibly we want a specific autoload declaration. As all
+%   this only affects the Prolog libraries,   we can always change this.
+%   One option might be this, where `How`   is one of `true`, `false` or
+%   `warning`.
+%
+%      :- autoloadable(PI, How)
+
+nowarn_autoload(TargetModule, LoadModule:PI) :-
+    NoWarn = LoadModule:'$nowarn_autoload'(PI,TargetModule),
+    '$c_current_predicate'(_, NoWarn),
+    \+ '$get_predicate_attribute'(NoWarn, imported, _From),
+    call(NoWarn).
+
 
                  /*******************************
                  *             REQUIRE          *
