@@ -3,11 +3,12 @@
     Purpose: Provide index for autoload
 */
 
-index((foreach), 2, aggregate, aggregate).
 index((aggregate), 3, aggregate, aggregate).
 index((aggregate), 4, aggregate, aggregate).
 index((aggregate_all), 3, aggregate, aggregate).
 index((aggregate_all), 4, aggregate, aggregate).
+index((foldall), 4, aggregate, aggregate).
+index((foreach), 2, aggregate, aggregate).
 index((free_variables), 4, aggregate, aggregate).
 index((ansi_format), 3, ansi_term, ansi_term).
 index((ansi_get_color), 2, ansi_term, ansi_term).
@@ -166,8 +167,6 @@ index((check_config_files), 0, check_installation, check_installation).
 index((update_config_files), 0, check_installation, check_installation).
 index((test_installation), 0, check_installation, check_installation).
 index((test_installation), 1, check_installation, check_installation).
-index((check_old_last), 0, checklast, checklast).
-index((check_old_select), 0, checkselect, checkselect).
 index((format_to_codes), 3, codesio, codesio).
 index((format_to_codes), 4, codesio, codesio).
 index((write_to_codes), 2, codesio, codesio).
@@ -234,6 +233,9 @@ index((assertion), 1, prolog_debug, debug).
 index((expects_dialect), 1, prolog_dialect, dialect).
 index((exists_source), 1, prolog_dialect, dialect).
 index((source_exports), 2, prolog_dialect, dialect).
+index((mapdict), 2, dicts, dicts).
+index((mapdict), 3, dicts, dicts).
+index((mapdict), 4, dicts, dicts).
 index((dicts_same_tag), 2, dicts, dicts).
 index((dict_size), 2, dicts, dicts).
 index((dict_keys), 2, dicts, dicts).
@@ -268,6 +270,9 @@ index((syntax_error), 1, error, error).
 index((must_be), 2, error, error).
 index((is_of_type), 2, error, error).
 index((current_type), 3, error, error).
+index((catch), 4, exceptions, exceptions).
+index((exception), 2, exceptions, exceptions).
+index((exception_type), 2, exceptions, exceptions).
 index((explain), 1, prolog_explain, explain).
 index((explain), 2, prolog_explain, explain).
 index((fast_read), 1, fastrw, fastrw).
@@ -425,9 +430,9 @@ index((nb_rb_set_node_value), 2, nb_rbtrees, nb_rbtrees).
 index((empty_nb_set), 1, nb_set, nb_set).
 index((add_nb_set), 2, nb_set, nb_set).
 index((add_nb_set), 3, nb_set, nb_set).
-index((gen_nb_set), 2, nb_set, nb_set).
 index((size_nb_set), 2, nb_set, nb_set).
 index((nb_set_to_list), 2, nb_set, nb_set).
+index((gen_nb_set), 2, nb_set, nb_set).
 index((contains_term), 2, occurs, occurs).
 index((contains_var), 2, occurs, occurs).
 index((free_of_term), 2, occurs, occurs).
@@ -569,6 +574,9 @@ index((format_types), 2, prolog_format, prolog_format).
 index((prolog_history), 1, prolog_history, prolog_history).
 index((jiti_list), 0, prolog_jiti, prolog_jiti).
 index((jiti_list), 1, prolog_jiti, prolog_jiti).
+index((jiti_suggest_modes), 1, prolog_jiti, prolog_jiti).
+index((jiti_suggest_modes), 0, prolog_jiti, prolog_jiti).
+index((setup_prolog_integer_grouping), 0, prolog_locale, prolog_locale).
 index((infer_meta_predicate), 2, prolog_metainference, prolog_metainference).
 index((inferred_meta_predicate), 2, prolog_metainference, prolog_metainference).
 index((pack_list_installed), 0, prolog_pack, prolog_pack).
@@ -591,6 +599,8 @@ index((profile), 2, prolog_profile, prolog_profile).
 index((show_profile), 1, prolog_profile, prolog_profile).
 index((profile_data), 1, prolog_profile, prolog_profile).
 index((profile_procedure_data), 2, prolog_profile, prolog_profile).
+index((qlf_make), 0, prolog_qlfmake, prolog_qlfmake).
+index((qlf_make), 1, prolog_qlfmake, prolog_qlfmake).
 index((prolog_read_source_term), 4, prolog_source, prolog_source).
 index((read_source_term_at_location), 3, prolog_source, prolog_source).
 index((prolog_open_source), 2, prolog_source, prolog_source).
@@ -691,7 +701,6 @@ index((atan2), 3, quintus, quintus).
 index((sign), 2, quintus, quintus).
 index((sqrt), 2, quintus, quintus).
 index((genarg), 3, quintus, quintus).
-index((mode), 1, quintus, quintus).
 index((no_style_check), 1, quintus, quintus).
 index((otherwise), 0, quintus, quintus).
 index((simple), 1, quintus, quintus).
@@ -886,11 +895,9 @@ index((thread_pool_destroy), 1, thread_pool, thread_pool).
 index((thread_create_in_pool), 4, thread_pool, thread_pool).
 index((current_thread_pool), 1, thread_pool, thread_pool).
 index((thread_pool_property), 2, thread_pool, thread_pool).
-index((thread_run_interactor), 0, thread_util, threadutil).
 index((threads), 0, thread_util, threadutil).
 index((join_threads), 0, thread_util, threadutil).
-index((interactor), 0, thread_util, threadutil).
-index((interactor), 1, thread_util, threadutil).
+index((with_stopped_threads), 2, thread_util, threadutil).
 index((thread_has_console), 0, thread_util, threadutil).
 index((attach_console), 0, thread_util, threadutil).
 index((attach_console), 1, thread_util, threadutil).
@@ -903,6 +910,9 @@ index((tnodebug), 1, thread_util, threadutil).
 index((tprofile), 1, thread_util, threadutil).
 index((tbacktrace), 1, thread_util, threadutil).
 index((tbacktrace), 2, thread_util, threadutil).
+index((thread_run_interactor), 0, thread_util, threadutil).
+index((interactor), 0, thread_util, threadutil).
+index((interactor), 1, thread_util, threadutil).
 index((tty_clear), 0, tty, tty).
 index((tty_flash), 0, tty, tty).
 index((menu), 3, tty, tty).
